<?php

/*________________________________________________________

  (C) 2003--2005 Maxim Proskurnya,     max@axofiber.org.ru
  All rights reserved 
  ________________________________________________________

  XML.lib v0.3 - parser of the XML tags

  This software is freeware and without any warranty and
  functionability. This software is not for commercial
  purpose or sale. You are free to modify or redistribute
  this script free  of charge under the following 
  conditions: redistributions of source code must retain
  the above copyright notice and the current disclaimer.
  ________________________________________________________

  XML structure:
  
  $Element["ELEM"]  <-- Element type:
    TAG  --- XML-element
    TEXT --- PCDATA, no name and subelements are supplied
  $Element["NAME"]  <-- Element name  
  $Element["ATTRS"] <-- array of attributes
  $Element["BODY"]  <-- array of subelements

  Elements without children are marked with attribute "/"
  
  Multiple root elements are allowed in this version. So
  ReadXML method return an array of the XML-elements, and
  you should take the first (0) element to access root.
  ________________________________________________________
*/

function ParseLine ( $Line, & $Tags )
{
  $i = 0;
  $Line = trim ( $Line );
  $StartPos = strpos ( $Line, "<" );
  while ( ! ( $StartPos === false ) )
  {
    $EndPos = strpos ( $Line, ">", $StartPos );
    $NewStartPos = strpos ( $Line, "<", $EndPos );
    if ( $NewStartPos === false ) 
      // Input exceeds
      $Tags[$i++] = trim ( substr ( $Line, $StartPos, $EndPos - $StartPos + 1 ) );
    else
      $Tags[$i++] = trim ( substr ( $Line, $StartPos, $NewStartPos - $StartPos ) );
    $StartPos = $NewStartPos;
  }
}

function SkipBlanks ( &$Tag, &$Pos )
{
  while ( $Pos < strlen ( $Tag ) && IsDelimiter ( $Tag[$Pos] ) ) ++$Pos;
}

function IsDelimiter ( $char )
{
  return strpos ( " \n\r\t", $char ) !== false;
}
  
function ParseTag ( $Tag )
{
  $Text = "";
  if ( !empty ( $Tag ) )
  {
    $Pos = 0;
    $AttrQnt = 0;
    $Tag = trim ( $Tag );
    if ( $Tag[$Pos++] == "<" )
    {
      SkipBlanks ( $Tag, $Pos );
      // Read tag name
      $AttrValue = "";
      while ( $Pos < strlen ( $Tag ) && !IsDelimiter ( $Tag[$Pos] ) && $Tag[$Pos] != ">") 
        $AttrValue .= $Tag[$Pos++];
      if ( !empty ( $AttrValue ) )
        $Attrs["NAME"] = $AttrValue;
      while ( $Pos < strlen ( $Tag ) && $Tag[$Pos] != ">" ) 
      {
        $AttrName = "";
        $AttrValue = "";
        SkipBlanks ( $Tag, $Pos );
        // Read attr name
        while ( $Pos < strlen ( $Tag ) && !IsDelimiter ( $Tag[$Pos] ) && $Tag[$Pos] != "=" && $Tag[$Pos] != ">" )
        {
          $c = $Tag[$Pos++];
          $AttrName .= $c;
        }
        SkipBlanks ( $Tag, $Pos );
        // Skip equal
        if ( $Pos < strlen ( $Tag ) && $Tag[$Pos] == "=" ) $Pos++;
        SkipBlanks ( $Tag, $Pos );
        // Read attr value
        if ( $Pos < strlen ( $Tag ) && $Tag[$Pos] == '"' )
        {
          // Quoted attribute
          ++$Pos;
          while ( $Pos < strlen ( $Tag ) && $Tag[$Pos] != '"' && $Tag[$Pos] != ">" )
          {
            $c = $Tag[$Pos++];
            if ( $c != "\n" && $c != "\r" ) $AttrValue .= $c;
          }
        }
        else
          while ( $Pos < strlen ( $Tag ) && !IsDelimiter ( $Tag[$Pos] ) && $Tag[$Pos] != ">" )
          {
            $c = $Tag[$Pos++];
            if ( $c != "\n" && $c != "\r" ) $AttrValue .= $c;
          }
        if ( !empty ( $AttrName ) )
          $Attrs[$AttrName] = $AttrValue;
        // Skip quote
        if ( $Pos < strlen ( $Tag ) && $Tag[$Pos] == "\"" )
          $Pos++;
      }
      // Retrieve text
      if ( $Pos < strlen ( $Tag ) ) 
      {
        for ( $i = $Pos+1; $i < strlen ($Tag); ++$i )
          if ( $Tag[$i] != "\n" && $Tag[$i] != "\r" )
            $Text .= $Tag[$i];
        $Text = trim ( $Text );
        if ( !empty ( $Text ) )
          $Attrs["TEXT"] = $Text;
      }         
     }
  }
  return $Attrs;
};
  
function ReadXMLElement ( & $Tags, & $Attrs, & $LastText, & $i )
{
  $CurrentItem = -1;
  $LastText    = "";
//print $i . "<br>";
  if ( isset ( $Attrs ) )
  {
    $TagName = $Attrs["NAME"];
    $Element["ELEM"] = "TAG";
    $Element["NAME"] = $TagName;
    foreach ( $Attrs as $key => $Attr )
    {
      if ( $key == "/" )
        $Element["ATTRS"]["/"] = "closed";
      else if ( $key != "TEXT" && $key != "NAME" )
        $Element["ATTRS"][$key] = $Attr;
    }
    if ( !empty ( $Attrs["TEXT"] ) )
    {
      ++$CurrentItem;
      $Element["BODY"][$CurrentItem]["ELEM"] = "TEXT";
      $Element["BODY"][$CurrentItem]["BODY"] = $Attrs["TEXT"];
    }
    if ( !isset ( $Element["ATTRS"]["/"] ) )
    {
      $CanContinue = $i + 1 < count ( $Tags );
      while ( $CanContinue )
      {
        $Attrs = ParseTag ( $Tags[++$i] );
//print ">" . $i . "-". $Attrs["NAME"] . "=" . "/" . $TagName . "<br>";
        if ( $Attrs["NAME"] == "/" . $TagName ) 
        {
          $CanContinue = False;
          $LastText = empty ( $Attrs["TEXT"] ) ? "" : $Attrs["TEXT"];
        }
        else
        {
          $Element["BODY"][++$CurrentItem] = ReadXMLElement ( $Tags, $Attrs, $LastText, $i );
          if ( !empty ( $LastText ) )
          {
            $Element["BODY"][++$CurrentItem]["ELEM"] = "TEXT";
            $Element["BODY"][$CurrentItem]["BODY"] = $Attrs["TEXT"];
          }
          $CanContinue = $i + 1 < count ( $Tags );
        }
      }
    }
  }
  return $Element;
}

function ReadXML ( $MarkUp )
{
  $i = -1;
  $j = -1;
  $LastText = "";
  ParseLine ( $MarkUp, $Tags );
  while ( $i + 1 < count ( $Tags ) )
  {
    $Attrs = ParseTag ( $Tags[++$i] );
    $XML[++$j] = ReadXMLElement ( $Tags, $Attrs, $LastText, $i );
  }
	return $XML;
};

function PrintXML ( & $XML, $Level )
{
  $Result = "";
  $Indent = "";
  for ( $i = 0; $i < $Level; ++$i ) $Indent .= ". . ";
  if ( isset ( $XML["ELEM"] ) )
    if ( $XML["ELEM"]=="TEXT" )
      $Result .= $Indent . $XML["BODY"] . "<br>\n";
    else if ( $XML["ELEM"]=="TAG" )
    {
      $Result .= $Indent . "&lt;".$XML["NAME"];
      if ( !empty ($XML["ATTRS"]) )
        foreach ( $XML["ATTRS"] as $key => $Attr )
  	    {
  		  if ( $key != "/" ) 
  		  {
            if ( strpos ( $Attr, ' ' ) !== false )
  		      $Result .=  ' '.$key.'="'. str_replace ( '"', '""', $Attr ) . '"';
            else
  	          $Result .=  ' '.$key.'='. $Attr;
  		  }
        }
      if (!empty ($XML["BODY"]) )
      {
  	    $Result .= "&gt;<br>\n";
  		  foreach ( $XML["BODY"] as $Element )
  				$Result .= PrintXML ( $Element, $Level + 1 );
  	    $Result .= $Indent . "&lt;/".$XML["NAME"]."><br>\n";
  	  }
  	  else
  	    $Result .= " /&gt;<br>\n";		
    }
  return $Result;
}

?>